"""
Configs migration

Generation datetime: 2024-07-10 06:40:23.386320
"""

import ujson
from configs_core.migration import models
from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)
from sqlalchemy import insert, outerjoin, select, update

mh: DatabaseMigrationHelper

REVISION = "8d592efc"
DOWN_REVISION = "19230084"


limitationsToModify: list[LimitationToModify] = []

externalLimitationsToModify: list[ExternalLimitationToModify] = []


def getSetting(name: str) -> dict:
    """
    Get setting by name from db.
    Args:
        name: luna-handlers setting name
    Returns:
        settings as dict
    """
    record = mh.connection.execute(
        select(
            [models.Tag.tag, models.Setting.value],
            from_obj=outerjoin(models.Setting, models.Tag, models.Setting.id == models.Tag.id),
        ).where(models.Setting.name == name)
    ).cursor.fetchall()
    settings = {tag: ujson.loads(value) for tag, value in dict(record).items()}
    return settings


def setSetting(value: dict, name: str, tag: str = None):
    """
    Set setting by name.
    Args:
        value: settings to be set
        name: luna-handlers setting name
    """
    if tag is None:
        mh.connection.execute(
            update(models.Setting)
            .where(models.Setting.name == name and models.Setting.is_default == True)
            .values(value=ujson.dumps(value, indent=1))
        )
    else:
        settingQuery = insert(models.Setting).values(  # type: ignore
            value=ujson.dumps(value, indent=1), name=name, is_default=False
        )
        record = mh.connection.execute(settingQuery)
        tagQuery = insert(models.Tag).values(id=record.inserted_primary_key[0], name=name, tag=tag)
        mh.connection.execute(tagQuery)


def upgrade():
    """Upgrade configs to new revision"""

    monitoringSetting = getSetting("INFLUX_MONITORING")
    # limitations to remove
    mh.removeLimitation(limitationName="INFLUX_MONITORING")

    # limitations to add
    mh.addLimitation(
        limitationName="LUNA_MONITORING",
        validationSchema={
            "oneOf": [
                {
                    "properties": {
                        "storage_type": {"type": "string", "enum": ["influx"]},
                        "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                        "use_ssl": {"type": "integer", "enum": [0, 1]},
                        "flushing_period": {"type": "number"},
                        "host": {"type": "string"},
                        "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                        "organization": {"type": "string"},
                        "token": {"type": "string"},
                        "bucket": {"type": "string"},
                    },
                    "default": {
                        "storage_type": "influx",
                        "send_data_for_monitoring": 1,
                        "use_ssl": 0,
                        "flushing_period": 1,
                        "host": "127.0.0.1",
                        "port": 8086,
                        "organization": "luna",
                        "token": "secret-token",
                        "bucket": "luna_monitoring",
                    },
                    "required": [
                        "storage_type",
                        "send_data_for_monitoring",
                        "host",
                        "port",
                        "organization",
                        "token",
                        "bucket",
                    ],
                    "type": "object",
                    "additionalProperties": False,
                    "description": "Influx monitoring settings.",
                }
            ]
        },
        defaultValue={
            "storage_type": "influx",
            "send_data_for_monitoring": 1,
            "use_ssl": 0,
            "flushing_period": 1,
            "host": "127.0.0.1",
            "port": 8086,
            "bucket": "luna_monitoring",
            "organization": "luna",
            "token": "secret-token",
        },
        description="monitoring parameters",
        services=[
            "luna-accounts",
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-lambda",
            "luna-lambda-tasks-unit",
            "luna-lambda-unit",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-remote-sdk",
            "luna-sender",
            "luna-tasks",
            "luna-video-agent",
            "luna-video-manager",
        ],
        groupNames=["luna-platform"],
    )

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    for tag, settings in monitoringSetting.items():
        value = settings.copy()
        value["storage_type"] = "influx"
        setSetting(value=value, name="LUNA_MONITORING", tag=tag)


def downgrade():
    """Downgrade configs to previous revision"""

    monitoringSetting = getSetting("LUNA_MONITORING")

    # limitations to modify
    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to add
    mh.addLimitation(
        limitationName="INFLUX_MONITORING",
        validationSchema={
            "properties": {
                "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                "use_ssl": {"type": "integer", "enum": [0, 1]},
                "flushing_period": {"type": "number"},
                "host": {"type": "string"},
                "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "organization": {"type": "string"},
                "token": {"type": "string"},
                "bucket": {"type": "string"},
            },
            "default": {
                "send_data_for_monitoring": 1,
                "use_ssl": 0,
                "flushing_period": 1,
                "host": "127.0.0.1",
                "port": 8086,
                "organization": "luna",
                "token": "secret-token",
                "bucket": "luna_monitoring",
            },
            "required": [
                "send_data_for_monitoring",
                "host",
                "port",
                "organization",
                "token",
                "bucket",
            ],
            "type": "object",
            "additionalProperties": False,
            "description": "Monitoring settings.",
        },
        defaultValue={
            "send_data_for_monitoring": 1,
            "use_ssl": 0,
            "flushing_period": 1,
            "host": "127.0.0.1",
            "port": 8086,
            "bucket": "luna_monitoring",
            "organization": "luna",
            "token": "secret-token",
        },
        description="monitoring parameters",
        services=[
            "luna-accounts",
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-lambda",
            "luna-lambda-tasks-unit",
            "luna-lambda-unit",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-remote-sdk",
            "luna-sender",
            "luna-tasks",
            "luna-video-agent",
            "luna-video-manager",
        ],
        groupNames=["luna-platform"],
    )

    # limitations to remove
    mh.removeLimitation(limitationName="LUNA_MONITORING")

    for tag, settings in monitoringSetting.items():
        value = settings.copy()
        value.pop("storage_type")
        setSetting(value=value, name="INFLUX_MONITORING", tag=tag)
